/***********************************************************************
 * Module:  WebviewJavaScriptBridge.java
 * Author:  allen
 * Purpose: Defines the Class WebviewJavaScriptBridge
 ***********************************************************************/

package com.hexin.android.webviewjsinterface;

import java.io.IOException;
import java.io.InputStream;
import java.lang.ref.SoftReference;
import java.util.ArrayList;
import java.util.Hashtable;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.content.Context;
import android.webkit.JavascriptInterface;
import android.webkit.WebView;

/** @pdOid 71429740-906b-46ce-9a91-4ad78669d6e4 */
public class WebviewJavaScriptBridge extends JavaScriptInterfaceFactory {
	public static final String JSFILE_NAME = "WebViewJavascriptBridge.js";
	public static final String NATIVE_ACTION_NAME = "jsBridge";
	// 要注入的jsString
	private String mJsString = null;
	// 存储当前网页的映射关系(methodName-JSInterfaceInstance)的缓存
	private Hashtable<String, JavaScriptInterface> mJSInterfaceMappingCache = new Hashtable<String, JavaScriptInterface>();
	// 存储当前的WebView
	private SoftReference<WebView> mWebviewRef = null;
	// 单例模式的实例
	private static WebviewJavaScriptBridge mWebviewJavaScriptBridge = null;

	private WebviewJavaScriptBridge() {
	}

	public static synchronized WebviewJavaScriptBridge getInstance() {
		if (mWebviewJavaScriptBridge == null) {
			mWebviewJavaScriptBridge = new WebviewJavaScriptBridge();
		}
		return mWebviewJavaScriptBridge;
	}

	/**
	 * 清空interface列表
	 * 
	 * 
	 * @pdOid 6e82a796-7abb-4e2f-9c45-de04b2d2a557
	 */
	private void clearInterfaces() {
		// 循环遍历列表，调用JavaScriptInterface.onWebviewPageRemove()方法
		if (mJSInterfaceMappingCache == null
				|| mJSInterfaceMappingCache.isEmpty()) {
			return;
		}
		JavaScriptInterface tempJavaScriptInterface;
		for (String key : mJSInterfaceMappingCache.keySet()) {
			tempJavaScriptInterface = mJSInterfaceMappingCache.get(key);
			if (tempJavaScriptInterface != null) {
				tempJavaScriptInterface.onInterfaceRemoved();
			}
		}
		mJSInterfaceMappingCache.clear();

	}

	/**
	 * 解析消息队列
	 * 
	 * [{"handlerName":"testObjcCallback","data":{"foo":"bar"},"callbackId":
	 * "cb_3_1400513009485"}]
	 * 
	 * 
	 * @param queueString
	 * @pdOid 13b6d81e-cef9-4847-95ab-166b14f4f685
	 */
	private ArrayList<MessageStruct> parseMessageQueue(String queueString) {
		if (queueString == null) {
			return null;
		}
		ArrayList<MessageStruct> resultList = new ArrayList<MessageStruct>();
		// 解析
		try {
			JSONArray jArray = new JSONArray(queueString);
			JSONObject tempObj = null;
			MessageStruct tempMessageStruct = null;
			for (int i = 0; i < jArray.length(); i++) {
				tempObj = jArray.getJSONObject(i);
				if (tempObj == null) {
					continue;
				}
				tempMessageStruct = new MessageStruct();
				// 获取methodName
				tempMessageStruct.methodName = tempObj.optString("handlerName");
				// 获取callBackId
				tempMessageStruct.callbackID = tempObj.optString("callbackId");
				// 获取数据
				tempMessageStruct.messageData = tempObj.optString("data");
				if (tempMessageStruct.methodName != null) {
					resultList.add(tempMessageStruct);
				}
			}
		} catch (JSONException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return resultList;

	}

	/**
	 * 获取JavaScriptInterface对象
	 * 
	 * @param messageStruct
	 * @pdOid 941d2237-c6fb-41b4-964d-7d7e3eeabd30
	 */
	private JavaScriptInterface getJavaScriptInterface(
			WebviewJavaScriptBridge.MessageStruct messageStruct) {
		if (messageStruct == null) {
			return null;
		}
		JavaScriptInterface jInterface = null;
		// 去缓存列表中取,取到之后返回
		jInterface = mJSInterfaceMappingCache.get(messageStruct.methodName);
		if (jInterface != null) {
			return jInterface;
		}
		// 取不到,使用反射去新建,新建完成之后,添加到缓存列表中,然后返回
		try {
			Class<JavaScriptInterface> clazz = (Class<JavaScriptInterface>) Class
					.forName(mJavaScriptInterfaces
							.get(messageStruct.methodName));
			if (clazz == null) {
				return null;
			}
			jInterface = clazz.newInstance();

		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InstantiationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (Exception e) {
			// TODO: handle exception
		}
		if (jInterface == null) {
			return null;
		}
		mJSInterfaceMappingCache.put(messageStruct.methodName, jInterface);
		return jInterface;
	}
	
	/**
	 * 向webview注册必须的native接口，这个方法在整个webview的生命周期里面注册一次就好
	 * @param webview 不能为空。如果为空的话，js和native交互的框架运行不了
	 */
	public void addJavascriptInterface(WebView webview){
	    if(webview == null){
	        return;
	    }
	    webview.addJavascriptInterface(new Object() {

            @JavascriptInterface
            public void onActionEvent(String messageString){
                WebviewJavaScriptBridge.getInstance().onActionEvent(messageString);
            }
        }, "android");
	}
	
	/**
	 * web页面显示出来，注册一些必须的依赖
	 * @param webview
	 */
	public void onWebviewShow(WebView webview) {
		// 记录当前处理的webview
		if (mWebviewRef != null) {
			mWebviewRef.clear();
			mWebviewRef = null;
		}
		mWebviewRef = new SoftReference<WebView>(webview);
	}

	/**
	 * 网页加载开始
	 * 
	 * 
	 * @param webview
	 * @param url
	 * @pdOid 9cbffa57-26a3-48ef-825b-71c720286864
	 */
	public void onWebviewPageLoadFinish() {
		// 合法性验证
		if (mWebviewRef == null || mWebviewRef.get() == null) {
			return;
		}
		Context context = mWebviewRef.get().getContext();
		if (mJsString == null) {
			// 加载要注入的jsString
			try {
				InputStream is = context.getAssets().open(JSFILE_NAME);
				byte[] buffer = new byte[is.available()];
				is.read(buffer);
				mJsString = new String(buffer);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		if (mJsString == null || "".equals(mJsString)) {
			// 异常情况，可能是没有将WebViewJavascriptBridge.js 复制到assets下
			return;
		}
		// 将上个网页的信息状态清掉，包括clear interface集合
		clearInterfaces();
		// 注入JS
		if(android.os.Build.VERSION.SDK_INT < 19){
			mWebviewRef.get().loadUrl("javascript: " + mJsString);
		}else{
			mWebviewRef.get().evaluateJavascript(mJsString, null);
		}
		
	}

	/** @pdOid 623cd42f-f5b1-48c3-8805-9a722579180e */
	public void onWebviewPageRemove(WebView webview) {
		// 将上个网页的信息状态清掉，包括clear interface集合
		clearInterfaces();
		if(mWebviewRef != null && webview == mWebviewRef.get()){
			// 断掉WEBVIEW的引用
			if (mWebviewRef != null) {
				mWebviewRef.clear();
				mWebviewRef = null;
			}
		}
	}

	/**
	 * @param webview
	 * @pdOid 783dbee0-7cdf-4d7c-852b-8f804158d7d1
	 */
	@JavascriptInterface
	public void onActionEvent(String messageString) {
		// 获取消息
		if (messageString == null) {
			return;
		}
		// 解析获取到的message队列
		ArrayList<MessageStruct> messageList = parseMessageQueue(messageString);
		if (messageList == null || messageList.size() == 0) {
			return;
		}
		if (mWebviewRef == null || mWebviewRef.get() == null) {
			return;
		}
		// 根据解析出来的方法，先从interfac列表中找方法，如果找到执行；如果没有找到，使用反射找到处理方法，添加到interface列表中，然后执行。
		MessageStruct tempMessageStruct = null;
		JavaScriptInterface tempInterface = null;
		for (int i = 0; i < messageList.size(); i++) {
			tempMessageStruct = messageList.get(i);
			tempInterface = getJavaScriptInterface(tempMessageStruct);
			if (tempInterface != null) {
			    if(tempMessageStruct.callbackID == null ||"".equals(tempMessageStruct.callbackID)){
			        tempInterface.onEventAction(mWebviewRef.get(),
                            tempMessageStruct.callbackID,
                            tempMessageStruct.methodName,
                            tempMessageStruct.messageData);
			    }else{
			        
			        tempInterface.onEventAction(mWebviewRef.get(),
			                tempMessageStruct.callbackID,
			                tempMessageStruct.messageData);
			    }
			}
		}
	}

	/** @pdOid c68cd79e-69d9-4802-a2e4-e9a5b000a9c9 */
	class MessageStruct {
		/** @pdOid 03e63440-88d0-46b8-babd-252489db0aaa */
		public String callbackID;
		/** @pdOid 30cdfc85-7a55-4969-90a7-6a44b614934e */
		public String messageData;
		/** @pdOid 8e695d9d-812b-4a3e-9ce9-d7535bf6f4ca */
		public String methodName;

	}

}